#include "inifileparser.h"
#include "debug.h"

ConfObject* INIFileParser::Parse(const std::string & file)
{
    FileIO* fileio = NULL;

    if((fileio = FileIO::OpenToRead(file)))
    {
		ConfObject* confobj = NULL;
		INIFileParser* parser = NULL;

		parser = new INIFileParser(fileio);
		confobj = parser->Parse();

		return confobj;
    }

    return NULL;
}

ConfObject* INIFileParser::Parse(FileIO* fileio)
{
   	ConfObject* confobj = NULL;
	INIFileParser* parser = NULL;

	if(fileio)
	{
		fileio->Reset();

		parser = new INIFileParser(fileio);
		confobj = parser->Parse();
	}

	return confobj;
}

ConfObject* INIFileParser::Parse()
{
	m_state = STATE_NULL;

	byte data[128] = {0};
    dword len = 0;

    while((len = m_fileio->Read(data, sizeof(data)-1)))
    {
    	byte* ptr = data;
        ptr[len] = 0;

        while(*ptr)
        {
        	*ptr = tolower(*ptr);
        	if(*ptr == '\n') m_linecount++;
        	if(*ptr == '\t') *ptr = ' ';
        	switch(m_state)
        	{
        		case STATE_NULL:
        			if(*ptr == '[') ParsePrepareNew();
        			break;

        		case STATE_SECTION:
        			ParseSection(*ptr);
        			break;

        		case STATE_SUBSECTION:
        			ParseSubSection(*ptr);
        			break;

        		case STATE_DATA_KEY:
        			ParseDataKey(*ptr);
        			break;

        		case STATE_DATA_VALUE:
        			ParseDataValue(*ptr);
        			break;
        	}

        	ptr++;
        }
    }
    Insert();

	return m_confobj;
}

void INIFileParser::ParsePrepareNew()
{
	m_subsection = "";
	m_section = "";
	m_key = "";
	m_value = "";
	m_state = STATE_SECTION;
}

void INIFileParser::ParseSection(byte b)
{
	if(b == ']') m_state = STATE_DATA_KEY;
    else if(b != '\r' && b != '\n') m_section += b;
}

void INIFileParser::ParseSubSection(byte b)
{
	if(b == '>') m_state = STATE_DATA_KEY;
    else if(b != '\r' && b != '\n') m_subsection += b;
}

void INIFileParser::ParseDataKey(byte b)
{
	if(m_key == "")
	{
		if(b == ' ' || b == '\r' || b == '\n') return;
		if(b == '<') { m_subsection = ""; m_state = STATE_SUBSECTION; return; }
		else if(b == '[') {	ParsePrepareNew(); return; }
	}
	else if(b == '\r' || b == '\n') { ParsePrepareNew(); Error(E_KEY); return; }
	else if(b == '=') { m_state = STATE_DATA_VALUE; return; }

	m_key += b;
}

void INIFileParser::ParseDataValue(byte b)
{
	if(m_value == "")
	{
		if(b == '\r' || b == '\n') { Insert(); Error(E_VALUE); return; }
		else if(b == ' ') return;
	}
	else if(b == ',')  { InsertMulti(); return; }
	else if(b == '\r') { return; }
	else if(b == '\n') { Insert(); return; }

	m_value += b;
}

void INIFileParser::InsertMulti()
{
	std::string key = m_key;

	Insert();

	m_key = key;
	m_state = STATE_DATA_VALUE;
}


void INIFileParser::Insert()
{
	truncspace(m_section);
	truncspace(m_subsection);
	truncspace(m_key);
	truncspace(m_value);

	if(m_confobj == NULL) m_confobj = new ConfObject();

	m_confobj->Insert(m_section, m_subsection, m_key, m_value);
	m_key = ""; m_value = ""; m_state = STATE_DATA_KEY;
}

void INIFileParser::Error(dword errnum)
{
	std::string errmsg = "";

	switch(errnum)
	{
		case S_OK:
			return;
			break;

		case E_SECTION:
			errmsg = "Error in section definition!";
			break;

		case E_SUBSECTION:
			errmsg = "Error in subsection definition!";
			break;

		case E_KEY:
			errmsg = "Error in key definition!";
			break;

		case E_VALUE:
			errmsg = "Error in value definition!";
			break;

		default:
			errmsg = "Unknown error!";
	}

	Debug::Warning("INIFileParser: %s [Line: %i]", errmsg.c_str(), m_linecount+1);

	// exit(1);
}

INIFileParser::INIFileParser(FileIO* fio) : m_fileio(fio), m_confobj(NULL)
{
	m_section = "";
	m_key = "";
	m_value = "";
	m_state = STATE_NULL;

	m_linecount = 0;
}

INIFileParser::INIFileParser()
{
}

INIFileParser::INIFileParser(INIFileParser & ifp)
{
}

INIFileParser::~INIFileParser()
{
}

