#include "wml_interface.h"
#include "debug.h"
#include "Ttos.h"
#include "fileio.h"

#include <string.h>
#include <iostream>
#include <sys/types.h>
#include <assert.h>

#define	DEFAULT_CONFIGFILE			"default.cfg"
#define DEFAULT_WEBOUTPUTPRENAME	"outputweb"
#define DEFAULT_OUTPUTPRENAME       "ausgabe"

static bool DEBUG = false;

void print_help ()
{
    std::cout << "Name\n\twmlgen - Program used to generate random WML-files" << std::endl;
    std::cout << "\nSynopsis\n\twmlgen [--help] [config file [output filename]] [--debug]" << std::endl;
    std::cout << "\nDescription\n\twmlgen produces random WML-files conform to the WML-DTD 1.3 published\n\tby the W3C." << std::endl;
    std::cout << "\twmlgen uses a config file -- either the given one or a default file --\n\tto produce randomized WML output." << std::endl;
    std::cout << "\tThe generated files can be stored in a file and used to test a\n\tWML-interpreter, e.g. a WML-browser, for stability or conformity to the\n\tstandard." << std::endl;
    std::cout << "\tIf persononalized files should be used, the first file must be the\n\tconfig file and the second the output filename." << std::endl;
    std::cout << "\nOptions\n\t--help\t\tPrints this help" << std::endl;
    std::cout << "\tconfig file\tSpecifies the config file that is used" << std::endl;
    std::cout << "\toutput filename\tSpecifies the name for the output files.\n\t\t\tThis option saves the generated document, the" << std::endl;
    std::cout << "\t\t\tconfig file and seed of the random generator to be able\n\t\t\tto reproduce the document." << std::endl;
    std::cout << "\t--debug\t\tPrints additional debug output" << std::endl;
    std::cout << "\nConfig file\n\tThe config file consits of up to four sections:" << std::endl;
    std::cout << "\n\t[global]\t  In this section rules for general options can be\n\t\t\t  specified, e.g. minimal length of the generated file." << std::endl;
    std::cout << "\t[dtd]\t\t  This section specifies the DTD the documents ares\n\t\t\t  conform to. All possible tags are definied in this\n\t\t\t  section." << std::endl;
    std::cout << "\t[direct_parents]  This section contains special rules for tags\n\t\t\t  that only apply if the tag has the specified direct\n\t\t\t  parent tag." << std::endl;
    std::cout << "\t[further_parents] This section contains special rules for\n\t\t\t  inderect child tags." << std::endl;
    std::cout << "\nExamples\n\twmlgen using standard configuration and output" << std::endl;
    std::cout << "\twmlgen <config file> using config file and standard output" << std::endl;
    std::cout << "\twmlgen <config file> <output filename> for additional output to files" << std::endl;
    std::cout << "\nFor further information please have a look at the man page (only available for\nLinux) or the manual." << std::endl;
}

bool EvalArgv(char* argv[], int argc, std::string & configfile, std::string & outputfilename)
{
	bool console = true;

	configfile = "";
	outputfilename = "";

	if(argc == 1)
	{
		configfile = DEFAULT_CONFIGFILE;
	}
	else
	{
		if(!pstrcmpi(argv[1], "--help") || !pstrcmpi(argv[1], "--h"))
		{
			print_help();
			exit(0);
		}
		else if(!pstrcmpi(argv[1], "--web") || !pstrcmpi(argv[1], "--w"))
		{
			if(argc < 3) exit(1);
			configfile  = argv[2];
			configfile += "/";
			outputfilename = configfile + DEFAULT_WEBOUTPUTPRENAME;

			if(argc > 3) configfile += argv[3];
			else configfile += DEFAULT_CONFIGFILE;
			console = false;
		}
		else
		{
			configfile = argv[1];
			if(argc <= 2)
			  outputfilename = DEFAULT_OUTPUTPRENAME;
			else
			  outputfilename = argv[2];
		}
	}

	if(!pstrcmpi(argv[argc-1], "--debug")) DEBUG = true;

	if(console) std::cout << "Using configuration file: [" << configfile << "]" << std::endl;

	return console;
}

std::string GetSeedFilename(const std::string & in)
{
	std::string temp;
	int pos = -1;

	if((pos = in.rfind(".")) > -1)
		temp = in.substr(0, pos);
	else
		temp = in;

	return temp + ".seed";
}

void BuildOutputFilenames(const std::string & out, std::string tm, std::string & cfgfile, std::string & seedfile, std::string & wmlfile)
{
	std::string temp = out + tm;
	cfgfile = temp + ".cfg";
	seedfile = temp + ".seed";
	wmlfile = temp + ".wml";
}

dword ReadSeedFromFile(const std::string & filename)
{
	FileIO* file = FileIO::OpenToRead(filename);
	if(file)
	{
		byte buffer[sizeof(dword)*4+1] = {0};
		file->Read(buffer, sizeof(buffer));
		buffer[sizeof(buffer)-1] = 0;
		delete file;
		return atol((char*)buffer);
	}
	return 0;
}


int main(int argc, char *argv[])
{
	// preparing
	dword predefinedseed = 0;
	std::string outputfilename;
	std::string configfilename;
	std::string seedfilename;
	std::string wmldoc;
	bool running_onconsole;
	int retval = 0;

	running_onconsole = EvalArgv(argv, argc, configfilename, outputfilename);
	seedfilename = GetSeedFilename(configfilename);
	predefinedseed = ReadSeedFromFile(seedfilename);

	Debug::Enable(DEBUG);

	//creating
	std::string timestamp = BuildTimestamp();
	WML_Interface* wml = NULL;

	if(predefinedseed)
		wml = new WML_Interface(configfilename, predefinedseed);
	else
		wml = new WML_Interface(configfilename);

	if(wml->IsReady())
	{
		wml->setTimestamp(timestamp);

		if(running_onconsole) std::cout << "Start Building WML Document" << std::endl;
  		wml->build_tree();
		if(running_onconsole) std::cout << "Finished Building WML Document" << std::endl;
		if(running_onconsole) std::cout << "Start Generating Output" << std::endl;
  		wmldoc = wml->output();
		if(running_onconsole) std::cout << "Finished Generating Output" << std::endl;

  		if(running_onconsole)
  		{
  			std::cout << "Output of WML Document" << std::endl << std::endl;
    		std::cout << wmldoc;
    	}
    	else
      		std::cout << timestamp  << std::endl;

		//Saving output if required
		if(outputfilename != "")
		{
			FileIO* of = NULL;
			std::string out_seedfile, out_cfgfile, out_wmlfile;

			BuildOutputFilenames(outputfilename, timestamp, out_cfgfile, out_seedfile, out_wmlfile);

			if((of = FileIO::OpenToWrite(out_wmlfile)))
			{
    			of->Write ((byte*)wmldoc.c_str(), (dword)wmldoc.size());
    			delete of;
    		}

    		if((of = FileIO::OpenToWrite(out_cfgfile)))
    		{
    			FileIO* cf = NULL;
    			if((cf = FileIO::OpenToRead(configfilename)))
    			{
    				of->Copy(cf);
    				delete cf;
    			}
    			delete of;
    		}

    		if((of = FileIO::OpenToWrite(out_seedfile)))
    		{
    			std::string strseed = Ttos<dword>(wml->get_seed());
    			of->Write ((byte*)strseed.c_str (), (dword)strseed.size());
    			delete of;
    		}
		}
	}
	else
	{
		std::cout << "The configuration file [" << configfilename << "] does not exist." << std::endl;
		std::cout << "Terminating." << std::endl;
		retval = 1;
	}

	delete wml;

	return retval;
}
